#!/usr/bin/env ruby
#
# This file is gererated by ruby-glade-create-template 1.1.4.
#
require 'libglade2'
require 'gtk2'
require 'socket'
#require 'iconv'

class ChatConnector
    def initialize(host, port, window)
        @host, @port, @window = host, port, window
    end
    
    def connect
        @socket = TCPSocket.new(@host, @port)
        @buf = @window.get_widget("messages").buffer
        @msg_stack = []
        
        # Tags
        font = Gtk::TextTag.new("font")
        font.font = "Monospace 10"
        color = Gtk::TextTag.new("color")
        color.foreground = "red"
        user = Gtk::TextTag.new("user")
        user.weight = Pango::FontDescription::WEIGHT_BOLD
        italic = Gtk::TextTag.new("italic")
        italic.style = Pango::FontDescription::STYLE_ITALIC
        gray = Gtk::TextTag.new("gray")
        gray.foreground = "gray"
        @buf.tag_table.add(font)
        @buf.tag_table.add(color)
        @buf.tag_table.add(user)
        @buf.tag_table.add(italic)
        @buf.tag_table.add(gray)
        
        
        # Nicklist
        @nicklist = @window.get_widget("nicklist")
        
        pix = Gtk::CellRendererPixbuf.new
        text = Gtk::CellRendererText.new

        @column = Gtk::TreeViewColumn.new
        @column.pack_start(pix, false)
        @column.set_cell_data_func(pix) do |column, cell, model, iter|
            cell.pixbuf = iter.get_value(0) 
        end
        
        @column.pack_start(text, true)
        @column.set_cell_data_func(text) do |column, cell, model, iter|
            cell.markup = iter.get_value(1) 
        end

        @nicklist.append_column(@column)
        store = Gtk::TreeStore.new(Gdk::Pixbuf, String)        
        @nicklist.set_model(store)

        # Listen-Thread
        @resp_thread = Thread.start {
            sleep 2
            start = true
            loop do
                msg = "" + @socket.gets.chomp
                msg = msg[1..msg.length] #if msg[0] == '\0'
                
                @msg_stack << msg
                update_nicklist if start
                start = false
                
                time = Time.now.strftime("%H:%M:%S")
                
                puts msg #DEBUG
                
                case msg
                    when /^\* / #Server msg
                        # Nicklist update?
                        if msg =~ /^\*.+(renamed to|join the chat|left the chat)/
                            @buf.insert(@buf.end_iter, "[#{time}] ", "font", "gray") # Time
                            @buf.insert(@buf.end_iter, "#{msg}\n", "font", "color", "italic")
                            update_nicklist
                        end
                    when /^([^ ]+): (.+)/ # User message
                        null, user, msg = msg.split(/^([^ ]+): (.+)/)
                        @buf.insert(@buf.end_iter, "[#{time}] ", "font", "gray") # Time
                        @buf.insert(@buf.end_iter, "#{user.rjust(18)}: ", "font", "user")
                        @buf.insert(@buf.end_iter, "#{msg}\n", "font")
                    else
                        @buf.insert(@buf.end_iter, "[#{time}] ", "font", "gray") # Time
                        @buf.insert(@buf.end_iter, "#{msg}\n", "font")
                end
                @window.get_widget("messages").scroll_to_iter(@buf.end_iter, 0.0, false, 0, 0)
            end
        }
    end
    
    def update_nicklist
        store = @nicklist.model
                
        Thread.start {
            stack_size = @msg_stack.size
            send_message("/users")
            sleep 2
            users = @msg_stack[stack_size..@msg_stack.size]
            store.clear
            users.each { |user|
                add_nick(user[2..user.size]) if user =~ /^\* / #BAAAD!
            }
        }
    end
    
    def send_nops
        Thread.start {
            loop do
                sleep 120
                send_message("/nop")
                puts "NOP"
            end
        }
    end
    
    def add_nick(name)
        nick = @nicklist.model.append(nil)
        nick[1] = name
        nick[0] = @nicklist.render_icon(Gtk::Stock::ABOUT, Gtk::IconSize::MENU, name)
    end
    
    def disconnect
        send_message("/quit")
        @socket.close
    end
    
    def send_message(msg)
        @socket.puts "#{msg}"
    end
    
    def change_nick(new_nick)
        send_message("/nick #{new_nick}")
    end
end

class ChatClientGuiGlade
  include GetText

  attr :glade
  
  def initialize(path_or_data, root = nil, domain = nil, localedir = nil, flag = GladeXML::FILE)
    bindtextdomain(domain, localedir, nil, "UTF-8")
    @glade = GladeXML.new(path_or_data, root, domain, localedir, flag) {|handler| method(handler)}
    @glade.get_widget("messages").buffer = Gtk::TextBuffer.new
    
    @connector = ChatConnector.new("aaron-mueller.de", 2009, @glade)
    @connector.connect
    @connector.send_nops
  end
  
  def on_input_key_press_event(widget, arg0)
    on_send_clicked(widget) if arg0.keyval == Gdk::Keyval::GDK_Return
  end
  def on_send_clicked(widget)
    input = @glade.get_widget("input")
    @connector.send_message(input.text) if input.text != ""
    input.text = ""
  end
  
  def on_nickchange_clicked(widget)
    dialog = Gtk::Dialog.new("New Nickanme", @glade.get_widget("main_window"),
        Gtk::Dialog::MODAL,
        [Gtk::Stock::OK, Gtk::Dialog::RESPONSE_ACCEPT],
        [Gtk::Stock::CANCEL, Gtk::Dialog::RESPONSE_REJECT])
    input_entry = Gtk::Entry.new
    input_entry.width_chars = 30
    dialog.vbox.add(input_entry)
    dialog.add_action_widget(input_entry, 1)
    dialog.show_all
    
    dialog.run { |resp|
        if resp == Gtk::Dialog::RESPONSE_ACCEPT and input_entry.text != ""
            @connector.change_nick(input_entry.text)
            @glade.get_widget("nick").text = input_entry.text + ": "
        end
        dialog.destroy
    }
  end
  def on_exit_clicked(widget)
    @connector.disconnect
    Gtk.main_quit
  end
end

# Main program
if __FILE__ == $0
  # Set values as your own application. 
  PROG_PATH = "chat_client_gui.glade"
  PROG_NAME = "Chat Client"
  ChatClientGuiGlade.new(PROG_PATH, nil, PROG_NAME)
  Gtk.main
end
